# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
import warnings
from typing import TYPE_CHECKING, Optional

from airflow.models import BaseOperator
from airflow.providers.tableau.operators.tableau import TableauOperator

if TYPE_CHECKING:
    from airflow.utils.context import Context


warnings.warn(
    """This operator is deprecated. Please use `airflow.providers.tableau.operators.tableau`.""",
    DeprecationWarning,
    stacklevel=2,
)


class TableauRefreshWorkbookOperator(BaseOperator):
    """
    This operator is deprecated. Please use `airflow.providers.tableau.operators.tableau`.

    Refreshes a Tableau Workbook/Extract

    .. seealso:: https://tableau.github.io/server-client-python/docs/api-ref#workbooks

    :param workbook_name: The name of the workbook to refresh.
    :type workbook_name: str
    :param site_id: The id of the site where the workbook belongs to.
    :type site_id: Optional[str]
    :param blocking: Defines if the job waits until the refresh has finished.
        Default: True.
    :type blocking: bool
    :param tableau_conn_id: The :ref:`Tableau Connection id <howto/connection:tableau>`
        containing the credentials to authenticate to the Tableau Server. Default:
        'tableau_default'.
    :type tableau_conn_id: str
    :param check_interval: time in seconds that the job should wait in
        between each instance state checks until operation is completed
    :type check_interval: float
    """

    def __init__(
        self,
        *,
        workbook_name: str,
        site_id: Optional[str] = None,
        blocking: bool = True,
        tableau_conn_id: str = 'tableau_default',
        check_interval: float = 20,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.workbook_name = workbook_name
        self.site_id = site_id
        self.blocking = blocking
        self.tableau_conn_id = tableau_conn_id
        self.check_interval = check_interval

    def execute(self, context: 'Context') -> str:
        """
        Executes the Tableau Extract Refresh and pushes the job id to xcom.

        :param context: The task context during execution.
        :type context: dict
        :return: the id of the job that executes the extract refresh
        :rtype: str
        """
        job_id = TableauOperator(
            resource='workbooks',
            method='refresh',
            find=self.workbook_name,
            match_with='name',
            site_id=self.site_id,
            tableau_conn_id=self.tableau_conn_id,
            blocking_refresh=self.blocking,
            check_interval=self.check_interval,
            task_id='refresh_workbook',
            dag=None,
        ).execute(context={})

        return job_id



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
This is an example dag that performs two refresh operations on a Tableau Workbook aka Extract. The first one
waits until it succeeds. The second does not wait since this is an asynchronous operation and we don't know
when the operation actually finishes. That's why we have another task that checks only that.
"""
from datetime import datetime, timedelta

from airflow import DAG
from airflow.providers.tableau.operators.tableau_refresh_workbook import TableauRefreshWorkbookOperator
from airflow.providers.tableau.sensors.tableau_job_status import TableauJobStatusSensor

with DAG(
    dag_id='example_tableau_refresh_workbook',
    dagrun_timeout=timedelta(hours=2),
    schedule_interval=None,
    start_date=datetime(2021, 1, 1),
    default_args={'site_id': 'my_site'},
    tags=['example'],
) as dag:
    # Refreshes a workbook and waits until it succeeds.
    task_refresh_workbook_blocking = TableauRefreshWorkbookOperator(
        workbook_name='MyWorkbook',
        blocking=True,
        task_id='refresh_tableau_workbook_blocking',
    )
    # Refreshes a workbook and does not wait until it succeeds.
    task_refresh_workbook_non_blocking = TableauRefreshWorkbookOperator(
        workbook_name='MyWorkbook',
        blocking=False,
        task_id='refresh_tableau_workbook_non_blocking',
    )
    # The following task queries the status of the workbook refresh job until it succeeds.
    task_check_job_status = TableauJobStatusSensor(
        job_id=task_refresh_workbook_non_blocking.output,
        task_id='check_tableau_job_status',
    )

    # Task dependency created via XComArgs:
    #   task_refresh_workbook_non_blocking >> task_check_job_status



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from typing import TYPE_CHECKING, Optional

from airflow.exceptions import AirflowException
from airflow.models import BaseOperator
from airflow.providers.tableau.hooks.tableau import (
    TableauHook,
    TableauJobFailedException,
    TableauJobFinishCode,
)

if TYPE_CHECKING:
    from airflow.utils.context import Context


RESOURCES_METHODS = {
    'datasources': ['delete', 'refresh'],
    'groups': ['delete'],
    'projects': ['delete'],
    'schedule': ['delete'],
    'sites': ['delete'],
    'subscriptions': ['delete'],
    'tasks': ['delete', 'run'],
    'users': ['remove'],
    'workbooks': ['delete', 'refresh'],
}


class TableauOperator(BaseOperator):
    """
    Execute a Tableau API Resource
    https://tableau.github.io/server-client-python/docs/api-ref

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:TableauOperator`

    :param resource: The name of the resource to use.
    :type resource: str
    :param method: The name of the resource's method to execute.
    :type method: str
    :param find: The reference of resource that will receive the action.
    :type find: str
    :param match_with: The resource field name to be matched with find parameter.
    :type match_with: Optional[str]
    :param site_id: The id of the site where the workbook belongs to.
    :type site_id: Optional[str]
    :param blocking_refresh: By default will be blocking means it will wait until it has finished.
    :type blocking_refresh: bool
    :param check_interval: time in seconds that the job should wait in
        between each instance state checks until operation is completed
    :type check_interval: float
    :param tableau_conn_id: The :ref:`Tableau Connection id <howto/connection:tableau>`
        containing the credentials to authenticate to the Tableau Server.
    :type tableau_conn_id: str
    """

    def __init__(
        self,
        *,
        resource: str,
        method: str,
        find: str,
        match_with: str = 'id',
        site_id: Optional[str] = None,
        blocking_refresh: bool = True,
        check_interval: float = 20,
        tableau_conn_id: str = 'tableau_default',
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.resource = resource
        self.method = method
        self.find = find
        self.match_with = match_with
        self.check_interval = check_interval
        self.site_id = site_id
        self.blocking_refresh = blocking_refresh
        self.tableau_conn_id = tableau_conn_id

    def execute(self, context: 'Context') -> str:
        """
        Executes the Tableau API resource and pushes the job id or downloaded file URI to xcom.
        :param context: The task context during execution.
        :type context: dict
        :return: the id of the job that executes the extract refresh or downloaded file URI.
        :rtype: str
        """
        available_resources = RESOURCES_METHODS.keys()
        if self.resource not in available_resources:
            error_message = f'Resource not found! Available Resources: {available_resources}'
            raise AirflowException(error_message)

        available_methods = RESOURCES_METHODS[self.resource]
        if self.method not in available_methods:
            error_message = f'Method not found! Available methods for {self.resource}: {available_methods}'
            raise AirflowException(error_message)

        with TableauHook(self.site_id, self.tableau_conn_id) as tableau_hook:

            resource = getattr(tableau_hook.server, self.resource)
            method = getattr(resource, self.method)

            resource_id = self._get_resource_id(tableau_hook)

            response = method(resource_id)

            job_id = response.id

            if self.method == 'refresh':
                if self.blocking_refresh:
                    if not tableau_hook.wait_for_state(
                        job_id=job_id,
                        check_interval=self.check_interval,
                        target_state=TableauJobFinishCode.SUCCESS,
                    ):
                        raise TableauJobFailedException(f'The Tableau Refresh {self.resource} Job failed!')

        return job_id

    def _get_resource_id(self, tableau_hook: TableauHook) -> str:

        if self.match_with == 'id':
            return self.find

        for resource in tableau_hook.get_all(resource_name=self.resource):
            if getattr(resource, self.match_with) == self.find:
                resource_id = resource.id
                self.log.info('Found matching with id %s', resource_id)
                return resource_id

        raise AirflowException(f'{self.resource} with {self.match_with} {self.find} not found!')
